// See COPYING for license information.
#include <fstream>
#include <iostream>
#include "defs.h"
#include "CConfigFile.h"

CConfigFile::CConfigFile()
{	// Load configuration file here, in the constructor.
	std::ifstream	_ifconf;
	_ifconf.open("/etc/zendo.conf", std::ios_base::in);
	while ( _ifconf.is_open() && !_ifconf.eof() ) {
		std::string	_line;
		getline(_ifconf, _line);
		// Check EOF here to prevent excess newlines from
		// cropping up at the end of the config file.
		if ( _ifconf.eof() )	break;
		_Contents.push_back(_line);
	}
	_ifconf.close();
}

CConfigFile::~CConfigFile()
{	// Save the config file.
	std::ofstream	_ofconf;
	_ofconf.open("/etc/zendo.conf", std::ios_base::out);
	for ( unsigned int i = 0; i < _Contents.size(); i++ )
		_ofconf << _Contents[i] << "\n";
	_ofconf.close();
}

std::vector<std::string> CConfigFile::GetValue(const std::string& valname)
{	// Get a value from the configuration file we have stored.
	std::vector<std::string>	_list;
	for ( unsigned int i = 0; i < _Contents.size(); i++ ) {
		if ( !_Contents[i].find(valname + "=") )
			_list.push_back(_Contents[i].substr(_Contents[i].find('=') + 1));
	}
	if ( _list.empty() ) {
		std::cerr << VT_COLOR_DOWNGRADE << "Error! Could not read '" << valname << "' from the config file!\n";
		std::cerr << "Expect the unexpected.\n" << VT_COLOR_NORMAL;
	}
	return _list;
}

void CConfigFile::SetValue(const std::string& valname, const std::vector<std::string>& valdata)
{	// Set a value in the stored configuration file.
	bool				_foundpos = false;
	std::vector<std::string>	_newlist;
	for ( unsigned int i = 0; i < _Contents.size(); i++ ) {
		if ( _Contents[i].find(valname + "=") )
			_newlist.push_back(_Contents[i]);
		else if ( !_foundpos ) {
			_foundpos = true;
			for ( unsigned int curval = 0; curval < valdata.size(); curval++ )
				_newlist.push_back(valname + "=" + valdata[curval]);
		}
	}
	if ( !_foundpos ) {	// It wasn't previously in the config file.
		for ( unsigned int curval = 0; curval < valdata.size(); curval++ )
			_newlist.push_back(valname + "=" + valdata[curval]);
	}
	_Contents = _newlist;
}
