// See COPYING for license information.

#include <algorithm>
#include "defs.h"
#include "CConfigFile.h"
#include "CDeepRemover.h"

CDeepRemover::CDeepRemover(std::vector<std::string>& packages)
{
	std::string	_choice;
	std::cout << VT_COLOR_DOWNGRADE << "Warning!  This function is somewhat dangerous." << VT_COLOR_NORMAL;
	std::cout << "\nPlease make sure you are connected to one of the official\n";
	std::cout << "mirrors for your distrobution, and that it supports dependencies.\n";
	while ( _choice.find('y') && _choice.find('n') ) {
		std::cout << "Are you sure you want to continue? [y/n]: ";
		std::cout.flush();
		std::getline(std::cin, _choice);
	}
	if ( !_choice.find('n') )	exit(0);

	// Convert to lowercase for matching reasons.
	for ( unsigned int i = 0; i < packages.size(); i++ )
		std::transform(packages[i].begin(), packages[i].end(), packages[i].begin(), tolower);

	// Load the database.
	CParser		_parser;
	package_map&	_db = _parser.Packages;

	// Get the blacklist from the config file.
	std::string	_blacklist = CConfigFile::Instance().GetValue("BlackList")[0];
	
	// Add all dependencies and subdependencies.
	for ( unsigned int i = 0; i < packages.size(); i++ ) {
		std::string	_strdeps = _db[packages[i]].dependencies;
		if ( !_strdeps.empty() )	_strdeps = _strdeps.substr(1);	// Chop off prefix comma.
		std::transform(_strdeps.begin(), _strdeps.end(), _strdeps.begin(), tolower);

		while ( (int)_strdeps.find(',') > 0 ) {
			std::string	_pkg = _strdeps.substr(0, _strdeps.find(','));
			std::transform(_pkg.begin(), _pkg.end(), _pkg.begin(), tolower);
			_strdeps = _strdeps.substr(_strdeps.find(',') + 1);

			if ( _db.find(_pkg) != _db.end() && !_db[_pkg].installed_version.empty() &&
					(int)_blacklist.find("," + _pkg + ",") == -1 ) {
				bool	_found = false;
				for ( unsigned int k = 0; k < packages.size(); k++ )
					if ( packages[k] == _pkg )	_found = true;
				if ( !check_removal_safety(_db[_pkg].name, _db, packages) )	_found = true;
				if ( !_found && dep_prompt(_db[packages[i]].name, _db[_pkg].name) )	packages.push_back(_pkg);
			}
		}
	}
}

bool CDeepRemover::check_removal_safety(const std::string& name, const package_map& db, const std::vector<std::string>& packages)
{	// Make sure this dep isn't needed by anything else.
	for ( package_map::const_iterator _iter = db.begin(); _iter != db.end(); ++_iter ) {
		if ( _iter->second.installed_version.empty() )	continue;
		
		bool	_found = false;	// Don't worry about this package if it is one of the removals.
		for ( unsigned int i = 0; i < packages.size(); i++ )
			if ( _iter->first == packages[i] )	_found = true;
		if ( _found )	continue;

		if ( (int)_iter->second.dependencies.find("," + name + ",") >= 0 )
			return false;	// Required by another package, thus not safe to remove.
	}
	return true;
}

bool CDeepRemover::dep_prompt(const std::string& package, const std::string& dep)
{	// Prompts the user about whether or not to remove a dep.
	std::cout << "Package '" << package << "' requires dependency '" << dep << "'.\n";

	std::string	_input;
	while ( _input.find('y') && _input.find('n') ) {
		std::cout << "Would you like to remove the dependency as well? [y/n] ";
		std::cout.flush();
		std::getline(std::cin, _input);
	}
	return _input.find('n');
}

